<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Jürgen Reinhold <jreinhold@freenet.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 *
 *
 * @package du_ranking
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
 *
 */
class Tx_DuRanking_Domain_Repository_TournamentRepository extends Tx_Extbase_Persistence_Repository {

	/**
	 * Finds all tournaments until a specific limit
	 *
	 * @param int $limit 
	 * @return array Matched tournaments
	 */
	public function findLimited($limit) {
		
		$query = $this->createQuery();
		$query->setLimit($limit);
		$query->setOrderings(array('tournamentDate' => Tx_Extbase_Persistence_QueryInterface::ORDER_DESCENDING));
		return $query->execute();
	}
	
	/**
	 * Finds all tournaments that meets the specified filter (demand).
	 *
	 * @param Tx_DuRanking_Domain_Model_Demand $demand 
	 * @return array Matched tournaments
	 */
	public function findDemanded(Tx_DuRanking_Domain_Model_Demand $demand = NULL) {
		
		$query = $this->createQuery();
		$constraints = array();
		if ($demand !== NULL) {
			if ($demand->getSeason() !== NULL) {
				$constraints[] = $query->equals('season', $demand->getSeason());
			}
			
			if ($demand->getLocation() !== NULL) {
				$constraints[] = $query->equals('location', $demand->getLocation());
			}
			
			if ($demand->getParticipant() !== NULL) {
				$constraints[] = $query->equals('positions.participant', $demand->getParticipant());
			}
			
			if (is_string($demand->getMonth()) && strlen($demand->getMonth()) > 0  && $demand->getMonth() != 0) {
				$month = (int) $demand->getMonth();
				$year = $this->findYear($demand->getSeason(), $month);
				$tsFirstDayOfMonth = mktime(0,0,0,$month,1,$year);	// The first day of the searched month as Timestamp
				$tsLastDayOfMonth = $this->findTimestampOfLastDay($tsFirstDayOfMonth);
				$constraints[] = $query->logicalAnd(
						$query->greaterThanOrEqual('tournament_date', $tsFirstDayOfMonth), 
						$query->lessThanOrEqual('tournament_date', $tsLastDayOfMonth)
					);
			}
		}
		$query->matching($query->logicalAnd($constraints));
		$query->setOrderings(array('tournamentDate' => Tx_Extbase_Persistence_QueryInterface::ORDER_DESCENDING));
		$result = $query->execute();
		
		return $result;
	}
	
	/**
	 * Finds all tournaments for the specified season.
	 *
	 * @param Tx_DuRanking_Domain_Model_Season $season 
	 * @return array Matched tournaments or NULL if $season = NULL
	 */
	public function findForSeason(Tx_DuRanking_Domain_Model_Season $season = NULL) {
		
		$result = NULL;
		$query = $this->createQuery();
		if ($season !== NULL) {
			$query->matching($query->equals('season', $season));
			$query->setOrderings(array('tournamentDate' => Tx_Extbase_Persistence_QueryInterface::ORDER_DESCENDING));
			$result = $query->execute();
		}
		return $result;
	}
	
	/**
	 * Finds all tournaments for the specified participant.
	 *
	 * @param Tx_DuRanking_Domain_Model_Person $participant 
	 * @param Tx_DuRanking_Domain_Model_Season $season
	 * @return array Matched tournaments or NULL if $participant = NULL
	 */
	public function findByPerson(Tx_DuRanking_Domain_Model_Person $participant = NULL, Tx_DuRanking_Domain_Model_Season $season) {
		
		$result = NULL;
		$query = $this->createQuery();
		if ($participant !== NULL) {
			$constraints = array();
			$constraints[] = $query->equals('season', $season);
			$constraints[] = $query->equals('positions.participant', $participant);
			$query->matching($query->logicalAnd($constraints));
			$result = $query->execute();
		}
		return $result;
	}
	
	/* Helper */
	/** 
	* Function to find the points for a given number of participants in statictable tupspublic.
	*
	* @param string $colName	name of the column, representing the tournament rank (i.e. number2 for Second place) 
	* @param int $posCount		number of positions/participants, the row
	* @return int $points		the points, result must exactly one value
	*
	*/
	public function getPointsFromTups($colName, $posCount){
		//SELECT $number FROM tx_duranking_static_tupspublic WHERE pers_count = $posCount
		$where = " pers_count = ".$posCount;
		//$where .= $this->cObj->enableFields('tx_duranking_static_tupspublic');
		$res = $GLOBALS["TYPO3_DB"]->exec_SELECTquery($colName,'tx_duranking_static_tupspublic',$where);
		while($row = $GLOBALS["TYPO3_DB"]->sql_fetch_assoc($res)){
			$points = $row[$colName];
		}
		return $points;
	}
	
	/* Statistic */
	
	/**
	 * Counts all tournaments for a given season
	 * 
	 * @param Tx_DuRanking_Domain_Model_Season $season 
	 * @return array tournamentCounts
	 */
	public function countTournaments(Tx_DuRanking_Domain_Model_Season $season) {
		$query = $this->createQuery();
		$query->matching($query->equals('season', $season));
		return $query->count();
	}
	
	/* intern */
	
	/** 
	* Function to find the last day of the month from the given first day as timestamp.
	*
	* @param int $tsFirstDayOfMonth the timestamp of the first day
	* @return int the timestamp of the last day
	*
	*/
	protected function findTimestampOfLastDay($tsFirstDayOfMonth) {
	    $splittedDate	  =    explode('.',date('t.m.Y',$tsFirstDayOfMonth));						//Find the last date of the month and separating it
	    $lastDayTimeStamp =    mktime(0,0,0,$splittedDate[1],$splittedDate[0],$splittedDate[2]);	//create time stamp of the last date of the give month
	    return $lastDayTimeStamp;
	}
	
	/** 
	* Function to find the year for a given month in the given season.
	*
	* @param Tx_DuRanking_Domain_Model_Season $season
	* @param int $month
	* @return int the year
	*
	*/
	protected function findYear(Tx_DuRanking_Domain_Model_Season $season, $month) {
		$seasonStartDate = $season->getBeginDate();
		$seasonStartMonth = (int)$seasonStartDate->format('n');
		$seasonStartYear = (int)$seasonStartDate->format('Y');
		
		if($seasonStartMonth > $month) {
			return $seasonStartYear + 1;
		}
	    return $seasonStartYear;
	}
	
}
?>